from .dual_VLC_player import B_DualVLCPlayer
from .download import b_download_m3u8


from .old_download import b_download_m3u8_old

__all__ = [
    'B_DualVLCPlayer',
    'b_download_m3u8',
    'b_download_m3u8_old'
]


import argparse
from pathlib import Path

def b_py2ipynb():
    parser = argparse.ArgumentParser()
    parser.add_argument("path", help="py文件")
    args = parser.parse_args()

    path = Path(args.path)
    if path.suffix != '.py':
        print("Error: 请输入.py文件路径")
        return

    from .Bconvert import b_py2ipynb
    b_py2ipynb(args.path)

def b_ipynb2py():
    parser = argparse.ArgumentParser()
    parser.add_argument("path", help="ipynb文件")
    args = parser.parse_args()

    path = Path(args.path)
    if path.suffix != '.ipynb':
        print("Error: 请输入.ipynb文件路径")
        return

    from .Bconvert import b_ipynb2py
    b_ipynb2py(args.path)

def b_str_finder():
    parser = argparse.ArgumentParser()
    parser.add_argument("--path", '-p')
    parser.add_argument("--string", '-s')
    args = parser.parse_args()

    from .Bfinder import b_str_finder
    b_str_finder(args.path, args.string)


from pdf2image import convert_from_path
from PIL import Image
import os
from concurrent.futures import ThreadPoolExecutor
from pathlib import Path
from fpdf import FPDF


def b_img2pdf(img_path, pdf_path):

    # 打开图片
    image = Image.open(img_path)

    # 转换为 RGB 模式（有些图片是 PNG 的 RGBA 格式）
    image = image.convert("RGB")

    # 保存为 PDF
    image.save(pdf_path)


def b_imgs2pdf(
    image_folder,
    output_pdf="output.pdf",
    sorted_key=lambda x: int(Path(x).stem),
):
    """
    :param image_folder:
    :param output_pdf:
    :param sorted_key:
    :return:

    >>> b_img2pdf(r'./文件夹1')

    """
    # 获取所有图片并排序
    images = [
        file for file in os.listdir(image_folder)
        if file.lower().endswith((".png", ".jpg", ".jpeg", ".bmp"))
    ]
    images = sorted(images, key=sorted_key)
    images = [os.path.join(image_folder, file) for file in images]


    if not images:
        print("没有找到图片文件！")
        return

    # 创建PDF对象
    pdf = FPDF()

    for image_file in images:
        # 打开图片文件
        img = Image.open(image_file)

        # 获取图片尺寸（转换为毫米）
        width, height = img.size
        width_mm = width * 0.264583
        height_mm = height * 0.264583

        # 添加PDF页面（使用图片原始尺寸）
        pdf.add_page(format=(width_mm, height_mm))

        # 将图片添加到PDF中
        pdf.image(image_file, 0, 0, width_mm, height_mm)

    # 保存PDF文件
    pdf.output(output_pdf)
    print(f"PDF已保存到: {output_pdf}")


def b_pdf2img(pdf_path, output_dir, format='png', dpi=200, cpu_use:int = None):
    '''
    :param pdf_path:
    :param output_dir:
    :param format: png, jpg, ...
    :param dpi: 越高越清晰, 125够用
    :param cpu_use: 采用核心数(不指定则采用总核心的四分之一)
    :return:
    '''
    pdf_path = Path(pdf_path)
    output_dir = Path(output_dir)
    pdf_name = pdf_path.stem
    if not pdf_path.is_file():
        print(f"路径 {pdf_path} 无法访问")
    os.makedirs(output_dir, exist_ok=True)

    cpu_count = os.cpu_count()
    if cpu_use is None:
        cpu_use = int(cpu_count / 4)
    print(f"[PDF to IMG] 当前CPU核心数{cpu_count}, 采用{cpu_use}个核心")
    print("[PDF to IMG] 正在处理数据中: ")
    images = convert_from_path(pdf_path, dpi=dpi, thread_count=cpu_use)

    def save_image(index, image):
        image.save(output_dir / f"{pdf_name}_{index + 1}.{format}", format.upper())

    with ThreadPoolExecutor() as executor:
        executor.map(save_image, range(len(images)), images)

    print(f"[PDF to IMG] {len(images)} 张 {format.upper()} 图片已保存至路径 {output_dir}")


from pptx import Presentation
from pptx.util import Inches
from PIL import Image
import os


def b_imgs2ppt(
        image_folder,
        output_ppt="output.pptx",
        base_image_path=None,
        sorted_key=lambda x: int(x.split(".")[0]),
        dpi=200
):
    '''
    将图片文件夹中的图片转换为 PPT 文件
    :param image_folder:
    :param output_ppt:
    :param base_image_path: 默认使用第一个图片作为基准图片
    :param sorted_key:
    :param dpi:
    :return:
    '''
    # 获取所有图片并排序
    images = sorted([
        file for file in os.listdir(image_folder)
        if file.lower().endswith((".png", ".jpg", ".jpeg", ".bmp", ".gif"))
    ])
    images = sorted(images, key=sorted_key)  # 按照数字大小排序

    prs = Presentation()
    # 获取基准图片的尺寸（像素）
    if base_image_path is None:
        base_image_path = os.path.join(image_folder, images[0])
    with Image.open(base_image_path) as img:
        width_px, height_px = img.size
    # 转换为英寸
    width_in = width_px / dpi
    height_in = height_px / dpi
    # 设置幻灯片尺寸（单位是英寸，需转成英寸再乘 Inches）
    prs.slide_width = Inches(width_in)
    prs.slide_height = Inches(height_in)

    # 遍历所有图片并插入到幻灯片中
    for img_name in images:
        img_path = os.path.join(image_folder, img_name)
        slide = prs.slides.add_slide(prs.slide_layouts[6])
        slide.shapes.add_picture(img_path, 0, 0, width=prs.slide_width, height=prs.slide_height)

    prs.save(output_ppt)
    print(f"PPT已保存到: {output_ppt}")



import json
from pathlib import Path

def b_ipynb2py(ipynb_path, py_path=None):
    '''

    :param ipynb_path:
    :param py_path:
    :return: None

    :examples:
    >>> b_ipynb2py('./test.ipynb')
    '''
    if py_path is None:
        py_path = Path(ipynb_path).parent / (Path(ipynb_path).stem + '.py')

    # 存在则询问是否覆盖
    if py_path.exists():
        ans = input(f"{py_path} 已存在. 是否覆盖? (y/n): ").strip().lower()
        if ans != 'y':
            print("Operation cancelled.")
            return

    with open(ipynb_path, mode='r', encoding='utf-8') as f:
        data = json.load(f)

    with open(py_path, mode='w', encoding='utf-8') as f:
        for i, x in enumerate(data['cells']):
            source = x['source']
            prefix = ''
            if source[0].strip() == '# test':
                prefix = '# '
            # 前缀
            f.write('#' * 24 + f'# cell {i + 1} ' + '#' * 25)
            f.write('\n\n')

            # 正文
            for sentence in source:
                f.write(prefix + sentence)

            # 后缀
            f.write('\n\n')

def b_py2ipynb(py_path, ipynb_path=None):
    '''
    如果遇到含`# cell`的注释，则分为一个块

    :param py_path:
    :param ipynb_path:
    :return: None

    :example:
    >>> b_py2ipynb('./example.py')
    '''
    if ipynb_path is None:
        ipynb_path = Path(py_path).parent / (Path(py_path).stem + '.ipynb')

    # 存在则询问是否覆盖
    if ipynb_path.exists():
        ans = input(f"{ipynb_path} 已存在. 是否覆盖? (y/n): ").strip().lower()
        if ans != 'y':
            print("Operation cancelled.")
            return

    with open(py_path, 'r', encoding='utf-8') as f:
        lines = f.readlines()

    cells = []
    current_cell = []
    start = True

    for line in lines:
        if ('# cell' in line) or ('#cell' in line):
            # 如果当前 cell 有内容，则保存为代码单元
            if current_cell:
                cells.append({
                    "cell_type": "code",
                    "metadata": {},
                    "outputs": [],
                    "execution_count": None,
                    "source": current_cell # 关键
                })
                current_cell = []
                start = True
            continue  # 注释行为 cell 分割标志，不写入 cell 中

        if start:
            if line.strip() == '':
                continue
            else:
                start = False
        current_cell.append(line)

    # 添加最后一个 cell（如果存在）
    if current_cell:
        cells.append({
            "cell_type": "code",
            "metadata": {},
            "outputs": [],
            "execution_count": None,
            "source": current_cell
        })

    notebook = {
        "cells": cells,
        "metadata": { # 元数据
            "kernelspec": { # 内核规范
                "display_name": "Python 3",
                "language": "python",
                "name": "python3"
            },
            "language_info": { # 关于编程语言的信息
                "name": "python",
                "version": "3.x"
            }
        },
        "nbformat": 4, # 格式版本号（版本 4 是当前的稳定版本）
        "nbformat_minor": 5
    }

    with open(ipynb_path, 'w', encoding='utf-8') as f:
        json.dump(notebook, f, indent=4)



if __name__ == '__main__':
    b_ipynb2py(r'E:\byzh_workingplace\byzh-rc-to-pypi\train.ipynb')
    # b_py2ipynb(r'E:\byzh_workingplace\byzh-rc-to-pypi\train.py')


import subprocess
import os
from pathlib import Path
from typing import Literal

CURRENT_DIR = Path(__file__).parent
PACKAGE_DIR = CURRENT_DIR.parent

FFMPEG_PATH = PACKAGE_DIR / 'bin/ffmpeg.exe'

def b_convert_video1(input_path: Path | str, output_path: Path | str):
    '''
    通过路径
    '''
    input_path, output_path = Path(input_path), Path(output_path)

    command = [
        FFMPEG_PATH,
        '-i', input_path,  # 输入文件
        '-c', 'copy',  # 拷贝编码，无需重新压缩（快）
        output_path
    ]

    try:
        subprocess.run(command, check=True)
        print(f"转换成功：{output_path}")
    except subprocess.CalledProcessError as e:
        print("转换失败：", e)

def b_convert_video2(video_path: Path | str, format: Literal['mp4', 'avi', 'ts', ...]):
    '''
    通过指定后缀
    '''
    input_path= Path(video_path)
    output_path = os.path.splitext(input_path)[0] + f'.{format}'
    output_path = Path(output_path)

    command = [
        FFMPEG_PATH,
        '-i', input_path,  # 输入文件
        '-c', 'copy',  # 拷贝编码，无需重新压缩（快）
        output_path
    ]

    try:
        subprocess.run(command, check=True)
        print(f"转换成功：{output_path}")
    except subprocess.CalledProcessError as e:
        print("转换失败：", e)





import os
from pathlib import Path
import chardet


def get_encoding(file_path):

    with open(file_path, 'rb') as f:
        result = chardet.detect(f.read())
        encoding = result['encoding']

    return encoding

def b_str_in_file(file_path, string, console_log=True):
    encoding = get_encoding(file_path)

    indexes = []
    try:
        with open(file_path, 'r', encoding=encoding) as f:
            for index, line in enumerate(f):
                if string in line:
                    indexes.append(index + 1)
    except UnicodeDecodeError:
        with open(file_path, 'r', encoding="utf-8") as f:
            for index, line in enumerate(f):
                if string in line:
                    indexes.append(index + 1)
    except Exception as e:
        print(f"Error: {e},\n\tencoding: {encoding},\n\tFilePath: {file_path}")


    if console_log:
        print(file_path, ' <-> ', indexes)

    if len(indexes) == 0:
        return False
    else:
        return indexes

INCLUDE_EXT = [
    '.py', '.html', '.css', '.js',
    '.md', '.txt',
    '.json', '.yaml', '.yml', '.ini',
]
def b_str_in_dir(dir_path, string, console_log=True, include_ext:list[str]=INCLUDE_EXT.copy()):
    '''
    找到文件夹内的指定后缀的文件中是否包含指定字符串，并返回包含该字符串的行号
    :param dir_path:
    :param string:
    :param include_ext:
    :param encoding:
    :return: list[[行号, 文件路径], ...]
    '''
    file_paths = []
    for root, dirs, files in os.walk(dir_path):
        for file in files:
            if Path(file).suffix not in include_ext:
                continue
            file_path = os.path.join(root, file)
            indexes = b_str_in_file(file_path, string, False)
            if indexes:
                file_paths.append([indexes, file_path])

    if console_log:
        for indexes, file_path in file_paths:
            print(file_path, ' <-> ', indexes)

    return file_paths

def b_str_finder(path, string, console_log=True):

    path = Path(path)
    if path.is_file():
        return b_str_in_file(path, string, console_log)
    elif path.is_dir():
        return b_str_in_dir(path, string, console_log)


if __name__ == '__main__':
    # lst = b_str_in_dir(r'E:\byzh_workingplace\byzh-rc-to-pypi\uploadToPypi_extra', 'Bos')
    # for i in lst:
    #     print(i)
    path = r'E:\byzh_study\byzh_code_note'
    b_str_finder(path, 'Trait')


import requests
import os
import shutil
import time
from pathlib import Path
from concurrent.futures import ThreadPoolExecutor, as_completed
from byzh_core.Btqdm import B_Tqdm

from ..Bffmpeg import b_merge_videos, b_convert_video1


def download_m3u8_file(m3u8_url, save_path='./playlist.m3u8', timeout=60):
    if os.path.exists(save_path):
        print(f"⚠️ 文件已存在，跳过下载：{save_path}")
        return
    try:
        response = requests.get(m3u8_url, timeout=timeout)
        response.raise_for_status()  # 抛出 HTTP 错误（如 404）

        # 保存内容到本地
        with open(save_path, 'w', encoding='utf-8', newline='') as f:
            f.write(response.text)

        print(f"✅ 下载成功，文件已保存到: {save_path}")

    except requests.exceptions.RequestException as e:
        print(f"❌ 下载失败：{e}")

def transform_m3u8_txt(m3u8_url, m3u8_m3u8_path, m3u8_txt_path='./playlist.txt'):
    # playlist.m3u8: 13个字符
    ts_url_prefix = m3u8_url[:-13]

    ts_list = []
    time_list = []

    # 保存内容到本地
    with open(m3u8_m3u8_path, 'r', encoding='utf-8') as f:
        for line in f:
            if line[0] != '#':
                ts_list.append(line.strip())
            elif 'EXTINF' in line:
                num = line.strip().split(':')[1][:-1]
                time_list.append(int(num))
    seconds = sum(time_list)

    print(f"片段数量：{len(ts_list)}")
    print(f"总时长：{seconds // 60 // 60}时{seconds // 60 % 60}分{seconds % 60}秒")

    with open(m3u8_txt_path, 'w', encoding='utf-8') as f:
        for i, ts in enumerate(ts_list):
            f.write(f'{ts_url_prefix + ts}')
            if i!= len(ts_list) - 1:
                f.write('\n')

    print(f"✅ 转换成功，文件已保存到: {m3u8_txt_path}")
    return len(ts_list)

def download_ts(my_tqdm, ts_url, save_path='./video_segment.ts', timeout=60):
    if os.path.exists(save_path):
        return
    try:
        response = requests.get(ts_url, stream=True, timeout=timeout)
        response.raise_for_status()

        with open(save_path, 'wb') as f: # wb用于写入视频流数据
            for chunk in response.iter_content(chunk_size=8192):
                if chunk:  # 跳过空块
                    f.write(chunk)
        my_tqdm.update(1)

    except requests.exceptions.RequestException as e:
        print(f"❌ 下载失败：{e}")

def download_ts_txt(my_tqdm, ts_txt_path, save_dir='./ts_segments', max_workers=16, timeout=60):
    with open(ts_txt_path, 'r', encoding='utf-8') as f:
        ts_list = f.readlines()

    os.makedirs(save_dir, exist_ok=True)

    def _download_task(i, ts_url):
        ts_url = ts_url.strip()
        save_path = os.path.join(save_dir, f"{i}.ts")
        download_ts(my_tqdm, ts_url, save_path, timeout=timeout)

    tasks = []
    with ThreadPoolExecutor(max_workers=max_workers) as executor:
        for i, ts_url in enumerate(ts_list):
            tasks.append(executor.submit(_download_task, i, ts_url))
        for future in as_completed(tasks):
            future.result()  # 抛出异常时会在此处触发


def b_download_m3u8(url, save_path, save_segments=True, max_workers=16, timeout=60):
    '''
    下载m3u8文件，然后合并为一个视频文件
    :param url: m3u8;链接
    :param save_path: 视频保存路径
    :param save_segments: 下载后是否保留ts文件
    :param max_workers: 线程数
    :param timeout: 最大请求时间
    :return:
    '''
    save_path = Path(save_path)
    file_name = save_path.name.split('.')[0]
    save_dir = save_path.parent / file_name

    os.makedirs(save_dir, exist_ok=True)

    # 下载playlist.m3u8
    m3u8_path1 = save_dir / 'playlist.m3u8'
    download_m3u8_file(url, m3u8_path1, timeout)
    # 转换为playlist.txt
    m3u8_path2 = save_dir / 'playlist.txt'
    length = transform_m3u8_txt(url, m3u8_path1, m3u8_path2)

    time.sleep(1)
    my_tqdm = B_Tqdm(total=length)

    # 下载ts文件
    segment_dir = save_dir / 'ts_segments'
    download_ts_txt(my_tqdm, m3u8_path2, segment_dir, max_workers, timeout)
    # 合并ts文件
    b_merge_videos(segment_dir, save_dir / 'video.ts')
    # 转换为指定格式
    b_convert_video1(save_dir / 'video.ts', save_path)

    time.sleep(1)
    if not save_segments:
        os.remove(m3u8_path1)
        os.remove(m3u8_path2)
        os.remove(save_dir / 'video.ts')
        shutil.rmtree(segment_dir)

    print(f"✅ 下载完成，视频已保存到: {save_path}")


# 层次分析法: AHP: Analytic Hierarchy Process

import numpy as np
from typing import Literal


def check_consistency(
        A: np.ndarray
):
    """
    一致性检验

    n个指标
    :param A: 形状(n, n)
    :return:
    """
    n = A.shape[0]

    # 计算所有 特征值, 特征向量
    eigen_values, eigen_vectors = np.linalg.eig(A)
    max_eigen_value = np.max(eigen_values)

    # 一致性指标
    CI = (max_eigen_value - n) / (n - 1)

    RI_list = [0, 0.0001, 0.52, 0.89, 1.12, 1.26, 1.36, 1.41, 1.46, 1.49, 1.52, 1.54, 1.56, 1.58, 1.59]
    # 查表即可
    RI = RI_list[n - 1]

    # 一致性比例
    CR = CI / RI

    if CR < 0.1:
        print(f"[通过] CR={np.abs(CR):.3f}<0.1 -> 判断矩阵A的一致性可以接受")
    else:
        raise ValueError("注意, CR>0.1, 所以判断矩阵A需要修改")


def mean(A: np.ndarray):
    n = A.shape[0]

    stand_A = A / np.sum(A, axis=0)

    sum_A = np.sum(stand_A, axis=1)

    weights = sum_A / n

    return weights  # (n,)


def geo_mean(A: np.ndarray):
    n = A.shape[0]

    prod_A = np.prod(A, axis=1)  # prod: product: 乘积
    prod_A = prod_A ** (1 / n)  # 开根号n

    weights = prod_A / np.sum(prod_A)  # 归一化

    return weights  # (n,)


def eigen(A: np.ndarray):
    n = A.shape[0]

    # 所有特征值eig_values, 所有特征向量eig_vectors
    # 注意: numpy.linalg.eig 返回的特征向量eig_vectors是以`列向量`形式排列的
    eig_values, eig_vectors = np.linalg.eig(A)
    # 去掉复数部分
    eig_values = np.abs(eig_values)
    eig_vectors = np.real(eig_vectors)

    # 找到最大特征值
    max_index = np.argmax(eig_values)
    # 及其对应的特征向量
    max_vector = eig_vectors[:, max_index]

    weights = max_vector / np.sum(max_vector)

    return weights  # (n,)


def b_analytic_hierarchy_process(
        A: np.ndarray,
        mode: Literal["mean", "geo_mean", "eigen"]
) -> np.ndarray:
    """
    层次分析法: AHP: analytic hierarchy process
    -> 得到各个指标的权重

    n个指标
    :param A: 判断矩阵, 形状(n, n)
    :param mode: 层次分析法的模式,
                包括"mean"(算术平均法), "geo_mean"(几何平均法), "eigen"(特征值法)
    :return: 各个指标的权重, 形状(n,)
    """
    # 一致性检验
    check_consistency(A)

    match mode:
        case "mean":  # 算术平均法
            weights = mean(A)
        case "geo_mean":  # 几何平均法
            weights = geo_mean(A)
        case "eigen":  # 特征值法
            weights = eigen(A)
        # 除此之外
        case _:
            raise ValueError("mode参数错误")

    return weights


b_AHP = b_analytic_hierarchy_process


if __name__ == '__main__':
    import numpy as np

    A = np.array([
        [1, 2, 3, 5],
        [1 / 2, 1, 1 / 2, 2],
        [1 / 3, 2, 1, 2],
        [1 / 5, 1 / 2, 1 / 2, 1]
    ])
    weights = b_AHP(A, "mean")
    print(weights)
    weights = b_AHP(A, "geo_mean")
    print(weights)
    weights = b_AHP(A, "eigen")
    print(weights)



import numpy as np
from typing import Literal

# L2归一化
def l2_normalize(A, mod:Literal["v", "h"]):
    n, m = A.shape

    # 行向量归一化
    if mod == "h":
        return A / np.sqrt(np.sum(A*A, axis=1))
    # 列向量归一化
    elif mod == "v":
        return A / np.sqrt(np.sum(A*A, axis=0))
    else:
        raise ValueError("axis should be 0 or 1")



from setuptools import setup, find_packages
import byzh_extra

setup(
    name='byzh_extra',
    version=byzh_extra.__version__,
    author="byzh_rc",
    description="基于byzh_core的扩展包",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'byzh_core==0.0.4.0', # !!!!!
        'python-pptx',
        'pdf2image',
        'chardet',
        'fpdf2'
    ],
    package_data={
        'byzh_extra': ['bin/*']
    },
    entry_points={
        "console_scripts": [
            "b_py2ipynb=byzh_extra.__main__:b_py2ipynb", # b_py2ipynb 路径
            "b_ipynb2py=byzh_extra.__main__:b_ipynb2py", # b_ipynb2py 路径
            "b_str_finder=byzh_extra.__main__:b_str_finder", # b_find_str -p path -s string
        ]
    },
)



from .ipynb_py import b_py2ipynb, b_ipynb2py
from .img_pdf import b_img2pdf, b_imgs2pdf, b_pdf2img
from .img_ppt import b_imgs2ppt
from .mp4_ts import b_ts2mp4

__all__ = [
    'b_py2ipynb', 'b_ipynb2py',
    'b_img2pdf', 'b_imgs2pdf', 'b_pdf2img',
    'b_imgs2ppt',
    'b_ts2mp4'
]


# 优劣解距离法: TOPSIS: Technique for Order Preference by Similarity an Ideal Solution
import numpy as np
from typing import Literal
from byzh_core.Butils import b_validate_params

from .math_utils import l2_normalize

def minTomax(x):  # 极小型处理方法
    max_x = np.max(x)
    array = max_x - x
    return array


def midTomax(x, bestx):  # 中间型处理方法
    delta_x = np.abs(x - bestx)
    M = max(delta_x)
    if M == 0:
        M = 1
    array = 1 - delta_x / M

    return array

def rangeTomax(x, lowx, highx):  # 区间型处理方法
    M = max(lowx - min(x), max(x) - highx)
    if M == 0:
        M = 1

    array = []
    for i in range(len(x)):
        if x[i] < lowx:
            array.append(1 - (lowx - x[i]) / M)
        elif x[i] <= highx:
            array.append(1)
        else:
            array.append(1 - (x[i] - highx) / M)
    array = np.array(array)

    return array

def max_better_normalization(
        A: np.ndarray,
        kind: list
):
    """
    矩阵正向化
    按列来
    :param A:
    :return:
    """
    def get_str(element):
        if isinstance(element, (tuple, list)):
            return element[0]
        elif isinstance(element, str):
            return element
        else:
            raise ValueError("kind参数错误")

    # 检查
    n, m = A.shape
    assert len(kind) == m, "kind参数长度与矩阵列数不一致"

    # 正向化
    result = A.copy().astype(float)
    for i in range(m):
        element = kind[i]
        # 极大型
        if get_str(element) == "max_better":
            continue
        # 极小型
        elif get_str(element) == "min_better":
            result[:, i] = minTomax(result[:, i])
        # 中间型
        elif get_str(element) == "mid_better":
            bestA = element[1]
            result[:, i] = midTomax(result[:, i], bestA)
        # 区间型
        elif get_str(element) == "interval_better":
            lowA, highA = element[1], element[2]
            result[:, i] = rangeTomax(result[:, i], lowA, highA)
        else:
            raise ValueError("kind参数错误")

    return result



@b_validate_params({
    "A": np.ndarray,
    "kind": {
        "type": list(
            Literal[
                "max_better",
                "min_better",
                ("mid_better", 'mid_num'),
                ("interval_better", 'low_num', 'high_num')
            ]
        ),
        "example": ["max_better", ("mid_better", 165), "min_better", ("interval_better", 90, 100)]
    }
})
def b_topsis(
        A: np.ndarray,
        kind: list
):
    """
    优劣解距离法: TOPSIS: Technique for Order Preference by Similarity an Ideal Solution
    -> 获得各候选项的得分

    n个候选项
    m个指标
    :param A:
    :param kind: 各指标的正向化类型
    :return:
    """
    n, m = A.shape

    # 正向化
    A = max_better_normalization(A, kind)

    # l2归一化(列)
    A = l2_normalize(A, "v")

    # 每个指标的 最大值, 最小值集
    A_max = np.max(A, axis=0)  # (m,)
    A_min = np.min(A, axis=0)  # (m,)

    # 每个候选项与之的距离
    d_z = np.sqrt(np.sum((A - A_max) ** 2, axis=1))  # (n,)
    d_f = np.sqrt(np.sum((A - A_min) ** 2, axis=1))  # (n,)

    # 每个候选项的得分
    s = d_f / (d_z + d_f)
    score = 100 * s / sum(s)  # 换为百分制

    return score

if __name__ == '__main__':

    A = np.array([
        [9, 10, 175, 120],
        [8, 7, 164, 80],
        [6, 3, 157, 90]
    ])

    score = b_topsis(
        A,
        ["max_better", "min_better", ("mid_better", 165), ("interval_better", 90, 100)]
    )

    print(score)